"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const schematics_1 = require("@angular-devkit/schematics");
const ts = require("typescript");
const ast_utils_1 = require("../utils/ast-utils");
const change_1 = require("../utils/change");
const find_module_1 = require("../utils/find-module");
const project_1 = require("../utils/project");
function addImportToNgModule(options) {
    return (host) => {
        options.path = project_1.getProjectPath(host, options);
        const modulePath = find_module_1.findModuleFromOptions(host, options);
        if (!modulePath) {
            return host;
        }
        if (!host.exists(modulePath)) {
            throw new Error('Specified module does not exist');
        }
        const text = host.read(modulePath);
        if (text === null) {
            throw new schematics_1.SchematicsException(`File ${modulePath} does not exist.`);
        }
        const sourceText = text.toString('utf-8');
        const source = ts.createSourceFile(modulePath, sourceText, ts.ScriptTarget.Latest, true);
        const importChanges = ast_utils_1.addImportToModule(source, modulePath, 'RoutingModule.forRoot()', 'angular-routing').shift();
        const changes = [
            ast_utils_1.insertImport(source, modulePath, 'RoutingModule', 'angular-routing'),
            importChanges,
        ];
        change_1.commitChanges(host, source.fileName, changes);
        return host;
    };
}
// Just return the tree
function default_1(options) {
    return schematics_1.chain([addImportToNgModule(options)]);
}
exports.default = default_1;
//# sourceMappingURL=index.js.map